/*
* serial.c - Steuerung COM
*
* Copyright (c) 2015 emtas GmbH
*-------------------------------------------------------------------
* $Id: $
*
*
*-------------------------------------------------------------------
*
*
*/

/********************************************************************/
/**
* \brief Serial handling
* This file contains the necessary serial init, read and write functions
*
*/


/* header of standard C - libraries
---------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <plib.h>
#include "gen_objectDict.h"

/* header of project specific types
---------------------------------------------------------------------------*/

/* constant definitions
---------------------------------------------------------------------------*/
#define DATA_BUFFER_LEN	50

/* local defined data types
---------------------------------------------------------------------------*/
#define UNSIGNED16	short int

/* list of external used functions, if not in headers
---------------------------------------------------------------------------*/

/* list of global defined functions
---------------------------------------------------------------------------*/

/* list of local defined functions
---------------------------------------------------------------------------*/

/* external variables
---------------------------------------------------------------------------*/

/* global variables
---------------------------------------------------------------------------*/

/* local defined variables
---------------------------------------------------------------------------*/
static volatile UNSIGNED16	trBufRdIdx = 0;
static volatile UNSIGNED16	trBufWrIdx = 0;
static volatile UNSIGNED16	recBufRdIdx = 0;
static volatile UNSIGNED16	recBufWrIdx = 0;
static char	trBuf[DATA_BUFFER_LEN];
static char	recBuf[DATA_BUFFER_LEN];


#define UART_MODULE_ID UART3 // PIM is connected to Explorer through UART3 module

#define DESIRED_BAUDRATE (9600) //The desired BaudRate

#define	GetPeripheralClock(SYS_FREQ)		(SYS_FREQ/(1 << OSCCONbits.PBDIV))
#define	GetInstructionClock(SYS_FREQ)		(SYS_FREQ)

/***************************************************************************/
/**
* \brief initSerial - init serial hardware and enable receive interrupt
*
*
* \return void
*
*/
void initSerial(
		int baudRate, unsigned long sysFreq
	)
{
	// Configure UART module, set buad rate, turn on UART, etc.
	UARTConfigure(UART_MODULE_ID, UART_ENABLE_PINS_TX_RX_ONLY);
	UARTSetFifoMode(UART_MODULE_ID, UART_INTERRUPT_ON_TX_BUFFER_EMPTY | UART_INTERRUPT_ON_RX_NOT_EMPTY);
	UARTSetLineControl(UART_MODULE_ID, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
	UARTSetDataRate(UART_MODULE_ID, GetPeripheralClock(sysFreq), baudRate);
	UARTEnable(UART_MODULE_ID, UART_ENABLE_FLAGS(UART_ENABLE | UART_PERIPHERAL | UART_RX | UART_TX));

	// Configure UART RX Interrupt
	INTEnable(INT_SOURCE_UART_RX(UART_MODULE_ID), INT_ENABLED);
	INTSetVectorPriority(INT_VECTOR_UART(UART_MODULE_ID), INT_PRIORITY_LEVEL_2);
	INTSetVectorSubPriority(INT_VECTOR_UART(UART_MODULE_ID), INT_SUB_PRIORITY_LEVEL_0);

	trBufRdIdx = 0;
	trBufWrIdx = 0;
	recBufRdIdx = 0;
	recBufWrIdx = 0;

	// Enable multi-vector interrupts
	INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);
	INTEnableInterrupts();
}


/***************************************************************************/
/**
* \brief UNSIGNED8 serialWrite - write data to serial
*
* Check, if transmit buffer has free space for whole data string
* Write data to transmit buffer
* Enable Transmit Interrupt
*
* \retVal 0
* 	transmission started
* \retVal -1
* 	not enough space at buffer
*
*/

int serialWrite(
		char		*pData,			/**< daten */
		UNSIGNED16	len				/**< daten länge */
	)
{
UNSIGNED16	i;
UNSIGNED16	idx;

	idx = trBufWrIdx;
	for (i = 0; i < len; i++)  {
		trBuf[idx] = *pData;
		idx++;
		pData++;
		if (idx == DATA_BUFFER_LEN)  {
			idx = 0;
		}
		/* free space at buffer ? */
		if (idx == trBufRdIdx)  {
			/* no, abort */
			return(-1);
		}
	}
	trBufWrIdx = idx;

	/* enable UART transmit int */
	INTEnable(INT_SOURCE_UART_TX(UART_MODULE_ID), INT_ENABLED);

	return(0);
}


/***************************************************************************/
/**
* \brief serialRead - read data from receive buffer, if complete string is available
*
* \retval 0
* 	no data available
* \retval >0
* 	string available, size is returned
* \retval <0
* 	string available, but until filled until maxBufLen
*
*/

int serialRead(
		char	*buf,
		int	maxBufLen
	)
{
static char	serialBuffer[DATA_BUFFER_LEN];
static UNSIGNED16	idx = 0;
UNSIGNED16	cnt = 0;

	/* Daten verfügbar ? */
	if (recBufWrIdx == recBufRdIdx)  {
		return;
	}

	/* get data from buffer and save it local */
	while (recBufWrIdx != recBufRdIdx)  {
		serialBuffer[idx] = recBuf[recBufRdIdx];
		recBufRdIdx++;
		if (recBufRdIdx == DATA_BUFFER_LEN)  {
			recBufRdIdx = 0;
		}

		if ((idx + 1) < DATA_BUFFER_LEN)  {
			idx++;
		}

		/* CR detected ? */
		if ((serialBuffer[idx - 1] == 0x0a) || (serialBuffer[idx - 1] == 0x0d)) {
			/* ok, Data available */
			cnt = idx;
			idx = 0;
			if (cnt > maxBufLen)  {
				cnt = maxBufLen;
				memcpy(buf, &serialBuffer[0], maxBufLen);
				return(0 - maxBufLen);
			} else {
				memcpy(buf, &serialBuffer[0], cnt);
				return(cnt);
			}
		}
	}
}

void addChar(const char c)
{
    static char	incomingBuffer[DATA_BUFFER_LEN];
    static UNSIGNED16 idx = 0;

    incomingBuffer[idx] = c;
    idx++;

    /* CR detected ? */
    if ((incomingBuffer[idx - 1] == 0x0a) || (incomingBuffer[idx - 1] == 0x0d)) {
            sscanf(incomingBuffer, "%d\n", &UART_Input_u32);
            // clear the array
            memset(&incomingBuffer[0], 0, sizeof(incomingBuffer));
            idx = 0;
    }
}
/***************************************************************************/
/**
* \brief USART1_IRQHandler - uart interrupt
*
*
*
* UART 1 interrupt handler, set at priority level 2
*
*/
void __ISR(_UART_3_VECTOR, ipl2) IntUart3Handler(void)
{
static UNSIGNED16	idx = 0;
	// Is this an RX interrupt?
	if (INTGetFlag(INT_SOURCE_UART_RX(UART_MODULE_ID)))
	{
		// Clear the RX interrupt Flag
		INTClearFlag(INT_SOURCE_UART_RX(UART_MODULE_ID));

		/* save data */
		//idx = recBufWrIdx;
                if(UARTReceivedDataIsAvailable(UART_MODULE_ID)) {
                    char c = UARTGetDataByte(UART_MODULE_ID);
                    addChar(c);
                }

		// Toggle LED to indicate UART activity
		mPORTAToggleBits(BIT_7);
	}

	// We don't care about TX interrupt
	if (INTGetFlag(INT_SOURCE_UART_TX(UART_MODULE_ID)))
	{
		INTClearFlag(INT_SOURCE_UART_TX(UART_MODULE_ID));

		if (trBufRdIdx != trBufWrIdx)  {

			/* copy data at transmit buffer */
			UARTSendDataByte(UART_MODULE_ID, trBuf[trBufRdIdx]);

			trBufRdIdx++;
			if (trBufRdIdx == DATA_BUFFER_LEN)  {
				trBufRdIdx = 0;
			}
		} else {
                         if(UARTTransmissionHasCompleted(UART_MODULE_ID))
                            			/* disable interrupt */
                            INTEnable(INT_SOURCE_UART_TX(UART_MODULE_ID), INT_DISABLED);
		}
	}
}
